"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const command_1 = require("@oclif/command");
const fs_1 = require("fs");
const lexer_1 = require("@pascal-psi/lexer");
const parser_1 = require("@pascal-psi/parser");
const interpreter_1 = require("@pascal-psi/interpreter");
const semantic_analyzer_1 = require("@pascal-psi/semantic-analyzer");
const symbol_1 = require("@pascal-psi/symbol");
const ast_1 = require("@pascal-psi/ast");
const library_1 = __importDefault(require("@pascal-psi/library"));
const chalk_1 = __importDefault(require("chalk"));
const cli_ux_1 = __importDefault(require("cli-ux"));
const path_1 = require("path");
function formatError(sourceCode, fileName, error) {
    const header = chalk_1.default.bold(fileName +
        (error.start.linePosition !== -1 && error.start.characterPosition !== -1
            ? `:${error.start.linePosition}:${error.start.characterPosition}:`
            : ':') +
        ' ' +
        chalk_1.default.redBright('Error:') +
        ' ' +
        error.message) + '\n\n';
    let codeLine = '';
    if (error.start.characterPosition === -1 ||
        error.start.linePosition === -1 ||
        error.end.characterPosition === -1 ||
        error.end.linePosition === -1) {
        codeLine = '<No line information was included>';
    }
    else if (error.start.linePosition === error.end.linePosition) {
        codeLine =
            sourceCode.split('\n')[error.start.linePosition - 1] +
                '\n' +
                ' '.repeat(error.start.characterPosition) +
                chalk_1.default.bold(chalk_1.default.blueBright(`^`.repeat(error.end.characterPosition - error.start.characterPosition || 1)));
    }
    else {
        codeLine =
            sourceCode.split('\n')[error.start.linePosition - 1] +
                '\n' +
                ' '.repeat(error.start.characterPosition) +
                chalk_1.default.bold(chalk_1.default.blueBright(`^`));
    }
    return header + codeLine + '\n';
}
class Psi extends command_1.Command {
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const { args, flags } = this.parse(Psi);
            let file = args.file;
            if (!file) {
                file = yield cli_ux_1.default.prompt('Please specify the file to interpret:');
            }
            let sourceCode = '';
            try {
                sourceCode = fs_1.readFileSync(file, 'utf8');
            }
            catch (error) {
                console.error('An error occured while attempting to read the source code file.\nPlease ensure the correct file path was provided and the appropriate permissions were set.\nError code: ' +
                    error.code);
                process.exit(1);
            }
            try {
                const lexer = new lexer_1.Lexer(sourceCode);
                const tree = new parser_1.Parser(lexer).run();
                const baseScope = new symbol_1.BaseSymbolScope('root');
                library_1.default(baseScope);
                new ast_1.RunnableChain(new semantic_analyzer_1.SymbolBuilder(tree, baseScope), new semantic_analyzer_1.TypeChecker(tree, baseScope)).run();
                const interpreter = new interpreter_1.Interpreter(tree, baseScope);
                interpreter.run();
                // console.log(
                //   (interpreter.scope.children as any).values().next().value.value,
                // );
            }
            catch (error) {
                if (error instanceof Error) {
                    throw error;
                }
                console.error(formatError(sourceCode, path_1.basename(file), error));
                console.error('Program execution halted with error(s).');
                process.exit(1);
            }
        });
    }
}
Psi.description = 'Interpret Pascal code';
Psi.flags = {
    version: command_1.flags.version({ char: 'v' }),
    help: command_1.flags.help({ char: 'h' }),
};
Psi.args = [
    {
        name: 'file',
        required: false,
        description: 'Input file path',
    },
];
module.exports = Psi;
//# sourceMappingURL=psi.js.map